#if defined __GNUC__ && __GNUC__ >= 14
#pragma GCC diagnostic warning "-Wimplicit-function-declaration"
#pragma GCC diagnostic warning "-Wincompatible-pointer-types"
#pragma GCC diagnostic warning "-Wint-conversion"
#pragma GCC diagnostic warning "-Wreturn-mismatch"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>

#include "Scorelist.h"
#include "UI.h"

#define NAMELEN 20
#define SCORES 10

typedef struct Score {
	char name[NAMELEN + 1];
	int level;
	int score;
} Score;

static Score scores[SCORES];

FILE *scorefile;

void
Scorelist_open() {
        gid_t realgid;

        /* Open the scoreboard file. */
        scorefile = fopen(SCOREFILE, "r+");

        /* Figure out who we really are. */
        realgid = getgid();

        /* This is where we drop our setuid/setgid privileges. */
        if (setresgid(-1, realgid, realgid) != 0) {
                perror("Could not drop setgid privileges.  Aborting.");
                exit(1);
        }
}

void
Scorelist_read() {
	int i;

	if (scorefile != NULL) {
		for (i = 0; i < SCORES; i++)
			fscanf(scorefile, "%20s%d%d\n", scores[i].name,
			       &scores[i].level, &scores[i].score);
	}
	else {
		for (i = 0; i < SCORES; i++) {
			strcpy(scores[i].name, "Anonymous");
			scores[i].level = 0;
			scores[i].score = 0;
		}
	}
}

void
Scorelist_write() {
	int i;
	if (scorefile == NULL)
		return;
	rewind(scorefile);
	for (i = 0; i < SCORES; i++)
		fprintf(scorefile, "%-*s %d %d\n", NAMELEN,
			scores[i].name, scores[i].level, scores[i].score);
	fclose(scorefile);
}

/*  Add new high score to list   */
void
Scorelist_recalc(const char *str, int level, int score) {
	int i;
	char tname[NAMELEN + 1];
	char *nl;

	if (scores[SCORES - 1].score >= score)
		return;
	for (i = SCORES - 1; i > 0; i--) {
		if (scores[i - 1].score < score) {
			strcpy (scores[i].name, scores[i - 1].name);
			scores[i].level = scores[i - 1].level;
			scores[i].score = scores[i - 1].score;
		}
		else
			break;
	}

	memset(tname, 0, sizeof(tname));
	if (str == NULL || str[0] == 0)
		strcpy(tname, "Anonymous");
	strncpy(tname, str, sizeof(tname) - 1);
	nl = strchr(tname,'\n');
	if (nl != NULL)
		*nl = 0;
	
	strcpy(scores[i].name, tname);
	scores[i].level = level;
	scores[i].score = score;
}

void
Scorelist_update() {
	char str[500];
	char temp[40];
	int i;
	strcpy(str, "High Scores:\n\n");
	sprintf(temp, "%-*s %6s %7s\n", NAMELEN, "Name", "Level", "Score");
	strcat(str, temp);
	for (i = 0; i < SCORES; i++) {
		sprintf(temp, "%-*s %6d %7d\n", NAMELEN,
			scores[i].name, scores[i].level, scores[i].score);
		strcat(str, temp);
	}
	UI_update_dialog(DIALOG_HIGHSCORE, str);
}

int
Scorelist_ishighscore(int val) {
	return (val > scores[SCORES - 1].score);
}
